@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of REANA.
-# Copyright (C) 2017, 2018 CERN.
+# Copyright (C) 2018, 2019, 2020, 2021 CERN.
 #
 # REANA is free software; you can redistribute it and/or modify it
 # under the terms of the MIT License; see LICENSE file for more details.
@@ -14,21 +14,22 @@ import sys
 import traceback
 
 import click
+from reana_commons.utils import click_table_printer
 
+from reana_client.printer import display_message
 from reana_client.api.utils import get_path_from_operation_id
 from reana_client.cli.utils import (
     add_access_token_options,
     add_pagination_options,
     add_workflow_option,
     check_connection,
-    filter_data,
-    parse_parameters,
+    format_data,
+    human_readable_or_raw_option,
+    parse_filter_parameters,
+    parse_format_parameters,
 )
-from reana_client.config import ERROR_MESSAGES, JSON, URL
+from reana_client.config import JSON, URL
 from reana_client.errors import FileDeletionError, FileUploadError
-from reana_client.utils import get_reana_yaml_file_path, workflow_uuid_or_name
-from reana_commons.utils import click_table_printer
-
 
 FILES_BLACKLIST = (".git/", "/.git/")
 
@@ -45,7 +46,7 @@ def files_group(ctx):
 @check_connection
 @click.option(
     "--format",
-    "_filter",
+    "_format",
     multiple=True,
     help="Format output according to column titles or column values. "
     "Use `<column_name>=<column_value>` format. For "
@@ -66,12 +67,31 @@ def files_group(ctx):
     default=None,
     help="Get URLs of output files.",
 )
+@click.option(
+    "--filter",
+    "filters",
+    multiple=True,
+    help="Filter results to show only files that match certain filtering "
+    "criteria such as file name, size or modification date."
+    "Use `--filter <columm_name>=<column_value>` pairs. "
+    "Available filters are `name`, `size` and `last-modified`.",
+)
 @click.argument("filename", metavar="SOURCE", nargs=1, required=False)
+@human_readable_or_raw_option
 @add_access_token_options
 @add_pagination_options
 @click.pass_context
 def get_files(
-    ctx, workflow, _filter, output_format, filename, access_token, page, size
+    ctx,
+    workflow,
+    _format,
+    filters,
+    output_format,
+    filename,
+    access_token,
+    page,
+    size,
+    human_readable_or_raw,
 ):  # noqa: D301
     """List workspace files.
 
@@ -82,7 +102,9 @@ def get_files(
 
     Examples: \n
     \t $ reana-client ls --workflow myanalysis.42 \n
-    \t $ reana-client ls --workflow myanalysis.42 'code/\*'
+    \t $ reana-client ls --workflow myanalysis.42 --human-readable \n
+    \t $ reana-client ls --workflow myanalysis.42 'data/*root*' \n
+    \t $ reana-client ls --workflow myanalysis.42 --filter name=hello
     """  # noqa: W605
     import tablib
     from reana_client.api.client import current_rs_api_client, list_files
@@ -91,13 +113,18 @@ def get_files(
     for p in ctx.params:
         logging.debug("{param}: {value}".format(param=p, value=ctx.params[p]))
 
-    if _filter:
-        parsed_filters = parse_parameters(_filter)
+    search_filter = None
+    headers = ["name", "size", "last-modified"]
+    if filters:
+        _, search_filter = parse_filter_parameters(filters, headers)
+    if _format:
+        parsed_format_filters = parse_format_parameters(_format)
     if workflow:
         logging.info('Workflow "{}" selected'.format(workflow))
         try:
-            response = list_files(workflow, access_token, filename, page, size)
-            headers = ["name", "size", "last-modified"]
+            response = list_files(
+                workflow, access_token, filename, page, size, search_filter
+            )
             data = []
             file_path = get_path_from_operation_id(
                 current_rs_api_client.swagger_spec.spec_dict["paths"], "download_file"
@@ -109,7 +136,11 @@ def get_files(
                         list(
                             map(
                                 str,
-                                [file_["name"], file_["size"], file_["last-modified"]],
+                                [
+                                    file_["name"],
+                                    file_["size"][human_readable_or_raw],
+                                    file_["last-modified"],
+                                ],
                             )
                         )
                     )
@@ -124,33 +155,30 @@ def get_files(
             for row in data:
                 tablib_data.append(row)
             if output_format == URL:
-                click.echo("\n".join(urls))
-            elif _filter:
-                tablib_data, filtered_headers = filter_data(
-                    parsed_filters, headers, tablib_data
+                display_message("\n".join(urls))
+            elif _format:
+                tablib_data, filtered_headers = format_data(
+                    parsed_format_filters, headers, tablib_data
                 )
                 if output_format == JSON:
-                    click.echo(json.dumps(tablib_data))
+                    display_message(json.dumps(tablib_data))
                 else:
                     tablib_data = [list(item.values()) for item in tablib_data]
                     click_table_printer(filtered_headers, filtered_headers, tablib_data)
             else:
                 if output_format == JSON:
-                    click.echo(tablib_data.export(output_format))
+                    display_message(tablib_data.export(output_format))
                 else:
-                    click_table_printer(headers, _filter, data)
+                    click_table_printer(headers, _format, data)
 
         except Exception as e:
             logging.debug(traceback.format_exc())
             logging.debug(str(e))
 
-            click.echo(
-                click.style(
-                    "Something went wrong while retrieving file list"
-                    " for workflow {0}:\n{1}".format(workflow, str(e)),
-                    fg="red",
-                ),
-                err=True,
+            display_message(
+                "Something went wrong while retrieving file list"
+                " for workflow {0}:\n{1}".format(workflow, str(e)),
+                msg_type="error",
             )
 
 
@@ -171,10 +199,10 @@ def download_files(
 ):  # noqa: D301
     """Download workspace files.
 
-    The `download` command allows to download workspace files. By default, the
-    files specified in the workflow specification as outputs are downloaded.
-    You can also specify the individual files you would like to download, see
-    examples below. Note that downloading directories is not yet supported.
+    The `download` command allows to download workspace files and directories.
+    By default, the files specified in the workflow specification as outputs
+    are downloaded. You can also specify the individual files you would like
+    to download, see examples below.
 
     Examples: \n
     \t $ reana-client download # download all output files \n
@@ -189,7 +217,9 @@ def download_files(
     if not filenames:
         reana_spec = get_workflow_specification(workflow, access_token)["specification"]
         if "outputs" in reana_spec:
-            filenames = reana_spec["outputs"].get("files") or []
+            filenames = []
+            filenames += reana_spec["outputs"].get("files", [])
+            filenames += reana_spec["outputs"].get("directories", [])
 
     if workflow:
         for file_name in filenames:
@@ -210,28 +240,23 @@ def download_files(
 
                 with open(outputs_file_path, "wb") as f:
                     f.write(binary_file)
-                click.secho(
+                display_message(
                     "File {0} downloaded to {1}.".format(file_name, output_directory),
-                    fg="green",
+                    msg_type="success",
                 )
             except OSError as e:
                 logging.debug(traceback.format_exc())
                 logging.debug(str(e))
-                click.echo(
-                    click.style(
-                        "File {0} could not be written.".format(file_name), fg="red"
-                    ),
-                    err=True,
+                display_message(
+                    "File {0} could not be written.".format(file_name),
+                    msg_type="error",
                 )
             except Exception as e:
                 logging.debug(traceback.format_exc())
                 logging.debug(str(e))
-                click.echo(
-                    click.style(
-                        "File {0} could not be downloaded: {1}".format(file_name, e),
-                        fg="red",
-                    ),
-                    err=True,
+                display_message(
+                    "File {0} could not be downloaded: {1}".format(file_name, e),
+                    msg_type="error",
                 )
 
 
@@ -284,59 +309,43 @@ def upload_files(ctx, workflow, filenames, access_token):  # noqa: D301
                     response = upload_to_server(workflow, filename, access_token)
                     for file_ in response:
                         if file_.startswith("symlink:"):
-                            click.echo(
-                                click.style(
-                                    "Symlink resolved to {}. Uploaded"
-                                    " hard copy.".format(file_[len("symlink:") :]),
-                                    fg="green",
-                                )
+                            display_message(
+                                "Symlink resolved to {}. "
+                                "Uploaded hard copy.".format(file_[len("symlink:") :]),
+                                msg_type="success",
                             )
                         else:
-                            click.echo(
-                                click.style(
-                                    "File {} was successfully "
-                                    "uploaded.".format(file_),
-                                    fg="green",
-                                )
+                            display_message(
+                                "File {} was successfully uploaded.".format(file_),
+                                msg_type="success",
                             )
                 except FileNotFoundError as e:
                     logging.debug(traceback.format_exc())
                     logging.debug(str(e))
-                    click.echo(
-                        click.style(
-                            "File {0} could not be uploaded: {0} does not"
-                            " exist.".format(filename),
-                            fg="red",
-                        ),
-                        err=True,
+                    display_message(
+                        "File {0} could not be uploaded: "
+                        "{0} does not exist.".format(filename),
+                        msg_type="error",
                     )
                     if "invoked_by_subcommand" in ctx.parent.__dict__:
                         sys.exit(1)
                 except FileUploadError as e:
                     logging.debug(traceback.format_exc())
                     logging.debug(str(e))
-                    click.echo(
-                        click.style(
-                            "Something went wrong while uploading {0}.\n{1}".format(
-                                filename, str(e)
-                            ),
-                            fg="red",
-                        ),
-                        err=True,
+                    display_message(
+                        "Something went wrong while uploading {0}.\n"
+                        "{1}".format(filename, str(e)),
+                        msg_type="error",
                     )
                     if "invoked_by_subcommand" in ctx.parent.__dict__:
                         sys.exit(1)
                 except Exception as e:
                     logging.debug(traceback.format_exc())
                     logging.debug(str(e))
-                    click.echo(
-                        click.style(
-                            "Something went wrong while uploading {}: \n{}".format(
-                                filename, str(e)
-                            ),
-                            fg="red",
-                        ),
-                        err=True,
+                    display_message(
+                        "Something went wrong while uploading {}: \n"
+                        "{}".format(filename, str(e)),
+                        msg_type="error",
                     )
                     if "invoked_by_subcommand" in ctx.parent.__dict__:
                         sys.exit(1)
@@ -356,7 +365,7 @@ def delete_files(ctx, workflow, filenames, access_token):  # noqa: D301
 
     Examples:\n
     \t $ reana-client rm -w myanalysis.42 data/mydata.csv \n
-    \t $ reana-client rm -w myanalysis.42 'code/\*'
+    \t $ reana-client rm -w myanalysis.42 'data/*root*'
     """  # noqa: W605
     from reana_client.api.client import delete_file
 
@@ -371,41 +380,29 @@ def delete_files(ctx, workflow, filenames, access_token):  # noqa: D301
                 freed_space = 0
                 for file_ in response["deleted"]:
                     freed_space += response["deleted"][file_]["size"]
-                    click.echo(
-                        click.style(
-                            "File {} was successfully deleted.".format(file_),
-                            fg="green",
-                        )
+                    display_message(
+                        f"File {file_} was successfully deleted.", msg_type="success"
                     )
                 for file_ in response["failed"]:
-                    click.echo(
-                        click.style(
-                            "Something went wrong while deleting {}.\n{}".format(
-                                file_, response["failed"][file_]["error"]
-                            ),
-                            fg="red",
-                        ),
-                        err=True,
+                    display_message(
+                        "Something went wrong while deleting {}.\n"
+                        "{}".format(file_, response["failed"][file_]["error"]),
+                        msg_type="error",
                     )
                 if freed_space:
-                    click.echo(
-                        click.style(
-                            "{} bytes freed up.".format(freed_space), fg="green"
-                        )
+                    display_message(
+                        f"{freed_space} bytes freed up.", msg_type="success"
                     )
             except FileDeletionError as e:
-                click.echo(click.style(str(e), fg="red"), err=True)
+                display_message(str(e), msg_type="error")
                 if "invoked_by_subcommand" in ctx.parent.__dict__:
                     sys.exit(1)
             except Exception as e:
                 logging.debug(traceback.format_exc())
                 logging.debug(str(e))
-                click.echo(
-                    click.style(
-                        "Something went wrong while deleting {}".format(filename),
-                        fg="red",
-                    ),
-                    err=True,
+                display_message(
+                    "Something went wrong while deleting {}".format(filename),
+                    msg_type="error",
                 )
                 if "invoked_by_subcommand" in ctx.parent.__dict__:
                     sys.exit(1)
@@ -436,38 +433,28 @@ def move_files(ctx, source, target, workflow, access_token):  # noqa: D301
         try:
             current_status = get_workflow_status(workflow, access_token).get("status")
             if current_status == "running":
-                click.echo(
-                    click.style(
-                        "File(s) could not be moved for running " "workflow", fg="red"
-                    ),
-                    err=True,
+                display_message(
+                    "File(s) could not be moved for running workflow", msg_type="error",
                 )
                 sys.exit(1)
             files = list_files(workflow, access_token)
             current_files = [file["name"] for file in files]
             if not any(source in item for item in current_files):
-                click.echo(
-                    click.style(
-                        "Source file(s) {} does not exist in "
-                        "workspace {}".format(source, current_files),
-                        fg="red",
-                    ),
-                    err=True,
+                display_message(
+                    "Source file(s) {} does not exist in "
+                    "workspace {}".format(source, current_files),
+                    msg_type="error",
                 )
                 sys.exit(1)
             mv_files(source, target, workflow, access_token)
-            click.echo(
-                click.style(
-                    "{} was successfully moved to {}.".format(source, target),
-                    fg="green",
-                )
+            display_message(
+                "{} was successfully moved to {}.".format(source, target),
+                msg_type="success",
             )
         except Exception as e:
             logging.debug(traceback.format_exc())
             logging.debug(str(e))
-            click.echo(
-                click.style("Something went wrong. {}".format(e), fg="red"), err=True
-            )
+            display_message("Something went wrong. {}".format(e), msg_type="error")
 
 
 @files_group.command("du")
@@ -476,14 +463,18 @@ def move_files(ctx, source, target, workflow, access_token):  # noqa: D301
 @add_access_token_options
 @click.option("-s", "--summarize", is_flag=True, help="Display total.")
 @click.option(
-    "-b", "--bytes", "block_size", flag_value="b", help="Print size in bytes."
-)
-@click.option(
-    "-k", "--kilobytes", "block_size", flag_value="k", help="Print size in kilobytes."
+    "--filter",
+    "filters",
+    multiple=True,
+    help="Filter results to show only files that match certain filtering "
+    "criteria such as file name or size."
+    "Use `--filter <columm_name>=<column_value>` pairs. "
+    "Available filters are `name` and `size`.",
 )
+@human_readable_or_raw_option
 @click.pass_context
 def workflow_disk_usage(
-    ctx, workflow, access_token, summarize, block_size
+    ctx, workflow, access_token, summarize, filters, human_readable_or_raw
 ):  # noqa: D301
     """Get workspace disk usage.
 
@@ -491,7 +482,8 @@ def workflow_disk_usage(
 
     Examples: \n
     \t $ reana-client du -w myanalysis.42 -s \n
-    \t $ reana-client du -w myanalysis.42 --bytes
+    \t $ reana-client du -w myanalysis.42 -s --human-readable \n
+    \t $ reana-client du -w myanalysis.42 --filter name=data/
     """
     from reana_client.api.client import get_workflow_disk_usage
 
@@ -499,24 +491,30 @@ def workflow_disk_usage(
     for p in ctx.params:
         logging.debug("{param}: {value}".format(param=p, value=ctx.params[p]))
 
+    search_filter = None
+    headers = ["size", "name"]
+    if filters:
+        _, search_filter = parse_filter_parameters(filters, headers)
     if workflow:
         try:
-            parameters = {"summarize": summarize, "block_size": block_size}
+            parameters = {"summarize": summarize, "search": search_filter}
             response = get_workflow_disk_usage(workflow, parameters, access_token)
-            headers = ["size", "name"]
+            if not response["disk_usage_info"]:
+                display_message("No files matching filter criteria.", msg_type="error")
+                sys.exit(1)
             data = []
             for disk_usage_info in response["disk_usage_info"]:
                 if not disk_usage_info["name"].startswith(FILES_BLACKLIST):
                     data.append(
-                        [disk_usage_info["size"], ".{}".format(disk_usage_info["name"])]
+                        [
+                            disk_usage_info["size"][human_readable_or_raw],
+                            ".{}".format(disk_usage_info["name"]),
+                        ]
                     )
             click_table_printer(headers, [], data)
         except Exception as e:
             logging.debug(traceback.format_exc())
             logging.debug(str(e))
-            click.echo(
-                click.style(
-                    "Disk usage could not be retrieved: \n{}".format(str(e)), fg="red"
-                ),
-                err=True,
+            display_message(
+                "Disk usage could not be retrieved: \n{}".format(e), msg_type="error",
             )
