@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of REANA.
-# Copyright (C) 2017, 2018 CERN.
+# Copyright (C) 2017, 2018, 2019, 2020, 2021 CERN.
 #
 # REANA is free software; you can redistribute it and/or modify it
 # under the terms of the MIT License; see LICENSE file for more details.
@@ -16,20 +16,29 @@ import traceback
 
 import click
 from jsonschema.exceptions import ValidationError
+from reana_commons.config import INTERACTIVE_SESSION_TYPES, REANA_COMPUTE_BACKENDS
+from reana_commons.errors import REANAValidationError
+from reana_commons.operational_options import validate_operational_options
+from reana_commons.utils import click_table_printer
+
 from reana_client.cli.files import get_files, upload_files
 from reana_client.cli.utils import (
     add_access_token_options,
+    add_access_token_options_not_required,
     add_pagination_options,
     add_workflow_option,
     check_connection,
-    filter_data,
+    format_data,
     format_session_uri,
+    get_formatted_progress,
+    human_readable_or_raw_option,
     key_value_to_dict,
-    parse_parameters,
+    parse_filter_parameters,
+    parse_format_parameters,
     requires_environments,
     validate_workflow_name,
 )
-from reana_client.config import ERROR_MESSAGES, TIMECHECK
+from reana_client.config import ERROR_MESSAGES, RUN_STATUSES, TIMECHECK
 from reana_client.printer import display_message
 from reana_client.utils import (
     get_reana_yaml_file_path,
@@ -40,10 +49,6 @@ from reana_client.utils import (
     validate_input_parameters,
     workflow_uuid_or_name,
 )
-from reana_commons.config import INTERACTIVE_SESSION_TYPES
-from reana_commons.errors import REANAValidationError
-from reana_commons.operational_options import validate_operational_options
-from reana_commons.utils import click_table_printer
 
 
 @click.group(help="Workflow management commands")
@@ -61,15 +66,18 @@ def workflow_execution_group(ctx):
 
 
 @workflow_management_group.command("list")
+@click.option(
+    "-w", "--workflow", default=None, help="List all runs of the given workflow.",
+)
 @click.option(
     "-s", "--sessions", is_flag=True, help="List all open interactive sessions."
 )
 @click.option(
     "--format",
-    "_filter",
+    "_format",
     multiple=True,
     help="Format output according to column titles or column values. "
-    "Use `<columm_name>=<column_value>` format. For "
+    "Use `<columm_name>=<column_value>` format. "
     "E.g. display workflow with failed status and named test_workflow "
     "`--format status=failed,name=test_workflow`.",
 )
@@ -93,42 +101,55 @@ def workflow_execution_group(ctx):
     count=True,
     help="Print out extra information: workflow id, user id, disk usage.",
 )
-@click.option(
-    "-b",
-    "--bytes",
-    "block_size",
-    flag_value="b",
-    help="Print workspace disk size in bytes (to be used with --verbose).",
-)
-@click.option(
-    "-k",
-    "--kilobytes",
-    "block_size",
-    flag_value="k",
-    help="Print workspace disk size in kilobytes (to be used with --verbose)",
-)
+@human_readable_or_raw_option
 @click.option(
     "--sort",
     "sort_columm_name",
     default="CREATED",
     help="Sort the output by specified column",
 )
+@click.option(
+    "--filter",
+    "filters",
+    multiple=True,
+    help="Filter workflow that contains certain filtering criteria. "
+    "Use `--filter <columm_name>=<column_value>` pairs. "
+    "Available filters are `name` and `status`.",
+)
+@click.option(
+    "--include-progress",
+    "include_progress",
+    is_flag=True,
+    default=None,
+    help="Include progress information of the workflows.",
+)
+@click.option(
+    "--include-workspace-size",
+    "include_workspace_size",
+    is_flag=True,
+    default=None,
+    help="Include size information of the workspace.",
+)
 @add_access_token_options
 @add_pagination_options
 @check_connection
 @click.pass_context
 def workflow_workflows(  # noqa: C901
     ctx,
+    workflow,
     sessions,
-    _filter,
+    _format,
     output_format,
     access_token,
     show_all,
     verbose,
-    block_size,
+    human_readable_or_raw,
     sort_columm_name,
     page,
     size,
+    filters,
+    include_progress,
+    include_workspace_size,
 ):  # noqa: D301
     """List all workflows and sessions.
 
@@ -149,20 +170,30 @@ def workflow_workflows(  # noqa: C901
     for p in ctx.params:
         logging.debug("{param}: {value}".format(param=p, value=ctx.params[p]))
     type = "interactive" if sessions else "batch"
-    if _filter:
-        parsed_filters = parse_parameters(_filter)
+
+    status_filter = None
+    search_filter = None
+    if filters:
+        filter_names = ["name", "status"]
+        status_filter, search_filter = parse_filter_parameters(filters, filter_names)
+    if _format:
+        parsed_format_filters = parse_format_parameters(_format)
     try:
-        if not verbose:
-            block_size = None
         response = get_workflows(
             access_token,
             type,
             verbose=bool(verbose),
-            block_size=block_size,
             page=page,
             size=size,
+            status=status_filter,
+            search=search_filter,
+            include_progress=include_progress,
+            include_workspace_size=include_workspace_size,
+            workflow=workflow,
         )
-        verbose_headers = ["id", "user", "size"]
+        verbose_headers = ["id", "user"]
+        workspace_size_header = ["size"]
+        progress_header = ["progress"]
         headers = {
             "batch": ["name", "run_number", "created", "started", "ended", "status"],
             "interactive": [
@@ -171,12 +202,18 @@ def workflow_workflows(  # noqa: C901
                 "created",
                 "session_type",
                 "session_uri",
+                "session_status",
             ],
         }
         if verbose:
             headers[type] += verbose_headers
+        if verbose or include_workspace_size:
+            headers[type] += workspace_size_header
+        if verbose or include_progress:
+            headers[type] += progress_header
         data = []
         for workflow in response:
+            workflow["size"] = workflow["size"][human_readable_or_raw]
             if workflow["status"] == "deleted" and not show_all:
                 continue
             name, run_number = get_workflow_name_and_run_number(workflow["name"])
@@ -190,13 +227,16 @@ def workflow_workflows(  # noqa: C901
                 )
             row = []
             for header in headers[type]:
-                if header == "started":
-                    header = "run_started_at"
-                elif header == "ended":
-                    header = "run_finished_at"
-                value = workflow.get(header)
+                value = None
+                if header in progress_header:
+                    value = get_formatted_progress(workflow.get("progress"))
+                elif header in ["started", "ended"]:
+                    _key = (
+                        "run_started_at" if header == "started" else "run_finished_at"
+                    )
+                    value = workflow.get("progress", {}).get(_key) or "-"
                 if not value:
-                    value = workflow.get("progress", {}).get(header) or "-"
+                    value = workflow.get(header)
                 row.append(value)
             data.append(row)
         sort_column_id = 2
@@ -215,29 +255,27 @@ def workflow_workflows(  # noqa: C901
         for row in data:
             tablib_data.append(row=row, tags=row)
 
-        if _filter:
-            tablib_data, filtered_headers = filter_data(
-                parsed_filters, headers[type], tablib_data
+        if _format:
+            tablib_data, filtered_headers = format_data(
+                parsed_format_filters, headers[type], tablib_data
             )
             if output_format:
-                click.echo(json.dumps(tablib_data))
+                display_message(json.dumps(tablib_data))
             else:
                 tablib_data = [list(item.values()) for item in tablib_data]
                 click_table_printer(filtered_headers, filtered_headers, tablib_data)
         else:
             if output_format:
-                click.echo(tablib_data.export(output_format))
+                display_message(tablib_data.export(output_format))
             else:
-                click_table_printer(headers[type], _filter, data)
+                click_table_printer(headers[type], _format, data)
 
     except Exception as e:
         logging.debug(traceback.format_exc())
         logging.debug(str(e))
-        click.echo(
-            click.style(
-                "Workflow list could not be retrieved: \n{}".format(str(e)), fg="red"
-            ),
-            err=True,
+        display_message(
+            "Workflow list could not be retrieved: \n{}".format(str(e)),
+            msg_type="error",
         )
 
 
@@ -295,7 +333,9 @@ def workflow_create(ctx, file, name, skip_validation, access_token):  # noqa: D3
         display_message("Workflow name cannot be a valid UUIDv4", msg_type="error")
     try:
         reana_specification = load_reana_spec(
-            click.format_filename(file), skip_validation
+            click.format_filename(file),
+            access_token=access_token,
+            skip_validation=skip_validation,
         )
         logging.info("Connecting to {0}".format(get_api_url()))
         response = create_workflow(reana_specification, name, access_token)
@@ -387,20 +427,21 @@ def workflow_start(
                     parsed_parameters["input_parameters"], original_parameters
                 )
             except REANAValidationError as e:
-                click.secho(e.message, err=True, fg="red")
+                display_message(e.message, msg_type="error")
                 sys.exit(1)
             except Exception as e:
-                click.secho(
+                display_message(
                     "Could not apply given input parameters: "
                     "{0} \n{1}".format(parameters, str(e)),
-                    err=True,
+                    msg_type="error",
                 )
         try:
             logging.info("Connecting to {0}".format(get_api_url()))
             response = start_workflow(workflow, access_token, parsed_parameters)
             current_status = get_workflow_status(workflow, access_token).get("status")
-            click.secho(
-                get_workflow_status_change_msg(workflow, current_status), fg="green"
+            display_message(
+                get_workflow_status_change_msg(workflow, current_status),
+                msg_type="success",
             )
             if follow:
                 while "running" in current_status:
@@ -408,14 +449,14 @@ def workflow_start(
                     current_status = get_workflow_status(workflow, access_token).get(
                         "status"
                     )
-                    click.secho(
+                    display_message(
                         get_workflow_status_change_msg(workflow, current_status),
-                        fg="green",
+                        msg_type="success",
                     )
                     if "finished" in current_status:
                         if follow:
-                            click.secho(
-                                "[INFO] Listing workflow output " "files...", bold=True
+                            display_message(
+                                "Listing workflow output files...", msg_type="info",
                             )
                             ctx.invoke(
                                 get_files,
@@ -429,11 +470,9 @@ def workflow_start(
         except Exception as e:
             logging.debug(traceback.format_exc())
             logging.debug(str(e))
-            click.echo(
-                click.style(
-                    "Cannot start workflow {}: \n{}".format(workflow, str(e)), fg="red"
-                ),
-                err=True,
+            display_message(
+                "Cannot start workflow {}: \n{}".format(workflow, str(e)),
+                msg_type="error",
             )
             if "invoked_by_subcommand" in ctx.parent.__dict__:
                 sys.exit(1)
@@ -539,30 +578,29 @@ def workflow_restart(
                 )
 
             except REANAValidationError as e:
-                click.secho(e.message, err=True, fg="red")
+                display_message(e.message, msg_type="error")
                 sys.exit(1)
             except Exception as e:
-                click.secho(
+                display_message(
                     "Could not apply given input parameters: "
                     "{0} \n{1}".format(parameters, str(e)),
-                    err=True,
+                    msg_type="error",
                 )
         try:
             logging.info("Connecting to {0}".format(get_api_url()))
             response = start_workflow(workflow, access_token, parsed_parameters)
             workflow = response["workflow_name"] + "." + str(response["run_number"])
             current_status = get_workflow_status(workflow, access_token).get("status")
-            click.secho(
-                get_workflow_status_change_msg(workflow, current_status), fg="green"
+            display_message(
+                get_workflow_status_change_msg(workflow, current_status),
+                msg_type="success",
             )
         except Exception as e:
             logging.debug(traceback.format_exc())
             logging.debug(str(e))
-            click.echo(
-                click.style(
-                    "Cannot start workflow {}: \n{}".format(workflow, str(e)), fg="red"
-                ),
-                err=True,
+            display_message(
+                "Cannot start workflow {}: \n{}".format(workflow, str(e)),
+                msg_type="error",
             )
             if "invoked_by_subcommand" in ctx.parent.__dict__:
                 sys.exit(1)
@@ -572,7 +610,7 @@ def workflow_restart(
 @add_workflow_option
 @click.option(
     "--format",
-    "_filter",
+    "_format",
     multiple=True,
     help="Format output by displaying only certain columns. "
     "E.g. --format name,status.",
@@ -589,7 +627,7 @@ def workflow_restart(
 @click.option("-v", "--verbose", count=True, help="Set status information verbosity.")
 @click.pass_context
 def workflow_status(  # noqa: C901
-    ctx, workflow, _filter, output_format, access_token, verbose
+    ctx, workflow, _format, output_format, access_token, verbose
 ):  # noqa: D301
     """Get status of a workflow.
 
@@ -675,9 +713,9 @@ def workflow_status(  # noqa: C901
         logging.debug("{param}: {value}".format(param=p, value=ctx.params[p]))
     if workflow:
         try:
-            if _filter:
-                parsed_filters = parse_parameters(_filter)
-                _filter = [item["column_name"] for item in parsed_filters]
+            if _format:
+                parsed_filters = parse_format_parameters(_format)
+                _format = [item["column_name"] for item in parsed_filters]
             response = get_workflow_status(workflow, access_token)
             headers = ["name", "run_number", "created", "status"]
             verbose_headers = ["id", "user", "command"]
@@ -698,24 +736,20 @@ def workflow_status(  # noqa: C901
                 for row in data:
                     tablib_data.append(row)
 
-                if _filter:
-                    tablib_data = tablib_data.subset(rows=None, cols=list(_filter))
+                if _format:
+                    tablib_data = tablib_data.subset(rows=None, cols=list(_format))
 
-                click.echo(tablib_data.export(output_format))
+                display_message(tablib_data.export(output_format))
             else:
-                click_table_printer(headers, _filter, data)
+                click_table_printer(headers, _format, data)
 
         except Exception as e:
             logging.debug(traceback.format_exc())
             logging.debug(str(e))
-            click.echo(
-                click.style(
-                    "Cannot retrieve the status of a workflow {}: \n{}".format(
-                        workflow, str(e)
-                    ),
-                    fg="red",
-                ),
-                err=True,
+            display_message(
+                "Cannot retrieve the status of a workflow {}: \n"
+                "{}".format(workflow, str(e)),
+                msg_type="error",
             )
 
 
@@ -760,11 +794,6 @@ def workflow_logs(
         "docker_img": "docker_img",
         "status": "status",
     }
-    compute_backends = {
-        "kubernetes": "Kubernetes",
-        "htcondor": "HTCondor",
-        "slurm": "Slurm",
-    }
     steps = []
     chosen_filters = dict()
 
@@ -777,38 +806,40 @@ def workflow_logs(
                 for f in filters:
                     key, value = f.split("=")
                     if key not in available_filters:
-                        click.echo(
-                            click.style(
-                                "Error: filter '{}' is not valid.\nAvailable filters are '{}'.".format(
-                                    key, "' '".join(sorted(available_filters.keys())),
-                                ),
-                                fg="red",
+                        display_message(
+                            "Filter '{}' is not valid.\n"
+                            "Available filters are '{}'.".format(
+                                key, "' '".join(sorted(available_filters.keys())),
                             ),
-                            err=True,
+                            msg_type="error",
                         )
                         sys.exit(1)
                     elif key == "step":
                         steps.append(value)
                     else:
                         # Case insensitive for compute backends
-                        value = (
-                            compute_backends[value.lower()]
-                            if key == "compute_backend"
-                            and value.lower() in compute_backends
-                            else value
-                        )
+                        if (
+                            key == "compute_backend"
+                            and value.lower() in REANA_COMPUTE_BACKENDS
+                        ):
+                            value = REANA_COMPUTE_BACKENDS[value.lower()]
+                        elif key == "status" and value not in RUN_STATUSES:
+                            display_message(
+                                "Input status value {} is not valid. ".format(value),
+                                msg_type="error",
+                            ),
+                            sys.exit(1)
                         chosen_filters[key] = value
             except Exception as e:
                 logging.debug(traceback.format_exc())
                 logging.debug(str(e))
-                click.echo(
-                    click.style(
-                        "Error: please provide complete --filter name=value pairs, for example --filter status=running.\nAvailable filters are '{}'.".format(
-                            "' '".join(sorted(available_filters.keys()))
-                        ),
-                        fg="red",
+                display_message(
+                    "Please provide complete --filter name=value pairs, "
+                    "for example --filter status=running.\n"
+                    "Available filters are '{}'.".format(
+                        "' '".join(sorted(available_filters.keys()))
                     ),
-                    err=True,
+                    msg_type="error",
                 )
                 sys.exit(1)
         try:
@@ -831,7 +862,7 @@ def workflow_logs(
                         del workflow_logs["job_logs"][job_id]
 
             if json_format:
-                click.echo(json.dumps(workflow_logs, indent=2))
+                display_message(json.dumps(workflow_logs, indent=2))
                 sys.exit(0)
             else:
                 from reana_client.cli.utils import output_user_friendly_logs
@@ -842,14 +873,10 @@ def workflow_logs(
         except Exception as e:
             logging.debug(traceback.format_exc())
             logging.debug(str(e))
-            click.echo(
-                click.style(
-                    "Cannot retrieve the logs of a workflow {}: \n{}".format(
-                        workflow, str(e)
-                    ),
-                    fg="red",
-                ),
-                err=True,
+            display_message(
+                "Cannot retrieve the logs of a workflow {}: \n"
+                "{}".format(workflow, str(e)),
+                msg_type="error",
             )
 
 
@@ -877,8 +904,18 @@ def workflow_logs(
     help="If set, try to pull remote environment image from registry to perform "
     "validation locally. Requires ``--environments`` flag. [default=False]",
 )
+@click.option(
+    "--server-capabilities",
+    is_flag=True,
+    default=False,
+    help="If set, check the server capabilities such as workspace validation. "
+    "[default=False]",
+)
+@add_access_token_options_not_required
 @click.pass_context
-def workflow_validate(ctx, file, environments, pull):  # noqa: D301
+def workflow_validate(
+    ctx, file, environments, pull, server_capabilities, access_token
+):  # noqa: D301
     """Validate workflow specification file.
 
     The `validate` command allows to check syntax and validate the reana.yaml
@@ -887,17 +924,25 @@ def workflow_validate(ctx, file, environments, pull):  # noqa: D301
     Examples: \n
     \t $ reana-client validate -f reana.yaml
     """
+    if server_capabilities:
+        if access_token:
+            check_connection(lambda: None)()
+        else:
+            display_message(ERROR_MESSAGES["missing_access_token"], msg_type="error")
+            ctx.exit(1)
     logging.debug("command: {}".format(ctx.command_path.replace(" ", ".")))
     for p in ctx.params:
         logging.debug("{param}: {value}".format(param=p, value=ctx.params[p]))
     try:
         load_reana_spec(
             click.format_filename(file),
+            access_token=access_token,
             skip_validate_environments=not environments,
             pull_environment_image=pull,
+            server_capabilities=server_capabilities,
         )
 
-    except ValidationError as e:
+    except (ValidationError, REANAValidationError) as e:
         logging.debug(traceback.format_exc())
         logging.debug(str(e))
         display_message(
@@ -906,6 +951,7 @@ def workflow_validate(ctx, file, environments, pull):  # noqa: D301
             ),
             msg_type="error",
         )
+        sys.exit(1)
     except Exception as e:
         logging.debug(traceback.format_exc())
         logging.debug(str(e))
@@ -913,6 +959,7 @@ def workflow_validate(ctx, file, environments, pull):  # noqa: D301
             "Something went wrong when trying to validate {}".format(file),
             msg_type="error",
         )
+        sys.exit(1)
 
 
 @workflow_execution_group.command("stop")
@@ -938,14 +985,14 @@ def workflow_stop(ctx, workflow, force_stop, access_token):  # noqa: D301
     Example: \n
     \t $ reana-client stop -w myanalysis.42 --force
     """
-    from reana_client.api.client import get_workflow_status, stop_workflow
+    from reana_client.api.client import stop_workflow
 
     if not force_stop:
-        click.secho(
+        display_message(
             "Graceful stop not implement yet. If you really want to "
             "stop your workflow without waiting for jobs to finish"
             " use: --force option",
-            fg="red",
+            msg_type="error",
         )
         raise click.Abort()
 
@@ -953,14 +1000,15 @@ def workflow_stop(ctx, workflow, force_stop, access_token):  # noqa: D301
         try:
             logging.info("Sending a request to stop workflow {}".format(workflow))
             stop_workflow(workflow, force_stop, access_token)
-            click.secho(get_workflow_status_change_msg(workflow, "stopped"), fg="green")
+            display_message(
+                get_workflow_status_change_msg(workflow, "stopped"), msg_type="success",
+            )
         except Exception as e:
             logging.debug(traceback.format_exc())
             logging.debug(str(e))
-            click.secho(
+            display_message(
                 "Cannot stop workflow {}: \n{}".format(workflow, str(e)),
-                fg="red",
-                err=True,
+                msg_type="error",
             )
 
 
@@ -1032,7 +1080,7 @@ def workflow_run(
     # set context parameters for subcommand
     ctx.invoked_by_subcommand = True
     ctx.workflow_name = ""
-    click.secho("[INFO] Creating a workflow...", bold=True)
+    display_message("Creating a workflow...", msg_type="info")
     ctx.invoke(
         workflow_create,
         file=file,
@@ -1040,14 +1088,14 @@ def workflow_run(
         skip_validation=skip_validation,
         access_token=access_token,
     )
-    click.secho("[INFO] Uploading files...", bold=True)
+    display_message("Uploading files...", msg_type="info")
     ctx.invoke(
         upload_files,
         workflow=ctx.workflow_name,
         filenames=None,
         access_token=access_token,
     )
-    click.secho("[INFO] Starting workflow...", bold=True)
+    display_message("Starting workflow...", msg_type="info")
     ctx.invoke(
         workflow_start,
         workflow=ctx.workflow_name,
@@ -1062,25 +1110,20 @@ def workflow_run(
 @click.option(
     "--include-all-runs",
     "all_runs",
-    count=True,
+    is_flag=True,
     help="Delete all runs of a given workflow.",
 )
 @click.option(
-    "--include-workspace", "workspace", count=True, help="Delete workspace from REANA."
-)
-@click.option(
-    "--include-records",
-    "hard_delete",
-    count=True,
-    help="Delete all records of workflow, including database entries and" " workspace.",
+    "--include-workspace",
+    "workspace",
+    is_flag=True,
+    help="Delete workspace from REANA.",
 )
 @add_workflow_option
 @add_access_token_options
 @check_connection
 @click.pass_context
-def workflow_delete(
-    ctx, workflow, all_runs, workspace, hard_delete, access_token
-):  # noqa: D301
+def workflow_delete(ctx, workflow, all_runs, workspace, access_token):  # noqa: D301
     """Delete a workflow.
 
     The `delete` command allows to remove workflow runs from the database and
@@ -1092,9 +1135,10 @@ def workflow_delete(
 
     Example: \n
     \t $ reana-client delete -w myanalysis.42 \n
-    \t $ reana-client delete -w myanalysis.42 --include-records
+    \t $ reana-client delete -w myanalysis.42 --include-all-runs \n
+    \t $ reana-client delete -w myanalysis.42 --include-workspace
     """
-    from reana_client.api.client import delete_workflow, get_workflow_status
+    from reana_client.api.client import delete_workflow
     from reana_client.utils import get_api_url
 
     logging.debug("command: {}".format(ctx.command_path.replace(" ", ".")))
@@ -1104,23 +1148,21 @@ def workflow_delete(
     if workflow:
         try:
             logging.info("Connecting to {0}".format(get_api_url()))
-            delete_workflow(workflow, all_runs, hard_delete, workspace, access_token)
+            delete_workflow(workflow, all_runs, workspace, access_token)
             if all_runs:
                 message = "All workflows named '{}' have been deleted.".format(
                     workflow.split(".")[0]
                 )
             else:
                 message = get_workflow_status_change_msg(workflow, "deleted")
-            click.secho(message, fg="green")
+            display_message(message, msg_type="success")
 
         except Exception as e:
             logging.debug(traceback.format_exc())
             logging.debug(str(e))
-            click.echo(
-                click.style(
-                    "Cannot delete workflow {} \n{}".format(workflow, str(e)), fg="red"
-                ),
-                err=True,
+            display_message(
+                "Cannot delete workflow {} \n{}".format(workflow, str(e)),
+                msg_type="error",
             )
 
 
@@ -1204,7 +1246,7 @@ def workflow_diff(
                     fg="yellow",
                 )
                 print_color_diff(content)
-        click.echo("")  # Leave 1 line for separation
+        display_message("")  # Leave 1 line for separation
         workspace_diff = json.loads(response.get("workspace_listing"))
         if workspace_diff:
             workspace_diff = workspace_diff.splitlines()
@@ -1218,12 +1260,9 @@ def workflow_diff(
     except Exception as e:
         logging.debug(traceback.format_exc())
         logging.debug(str(e))
-        click.echo(
-            click.style(
-                "Something went wrong when trying to get diff:\n{}".format(str(e)),
-                fg="red",
-            ),
-            err=True,
+        display_message(
+            "Something went wrong when trying to get diff:\n{}".format(str(e)),
+            msg_type="error",
         )
 
 
@@ -1277,6 +1316,9 @@ def workflow_open_interactive_session(
                 interactive_session_type,
                 interactive_session_configuration,
             )
+            display_message(
+                "Interactive session opened successfully", msg_type="success"
+            )
             click.secho(
                 format_session_uri(
                     reana_server_url=ctx.obj.reana_server_url,
@@ -1285,19 +1327,18 @@ def workflow_open_interactive_session(
                 ),
                 fg="green",
             )
-            click.echo(
-                "It could take several minutes to start the " "interactive session."
+            display_message(
+                "It could take several minutes to start the interactive session."
             )
         except Exception as e:
             logging.debug(traceback.format_exc())
             logging.debug(str(e))
-            click.secho(
+            display_message(
                 "Interactive session could not be opened: \n{}".format(str(e)),
-                fg="red",
-                err=True,
+                msg_type="error",
             )
     else:
-        click.secho("Cannot find workflow {}".format(workflow), fg="red", err=True)
+        display_message("Cannot find workflow {}".format(workflow), msg_type="error")
 
 
 @interactive_group.command("close")
@@ -1321,17 +1362,17 @@ def workflow_close_interactive_session(workflow, access_token):  # noqa: D301
         try:
             logging.info("Closing an interactive session on {}".format(workflow))
             close_interactive_session(workflow, access_token)
-            click.echo(
+            display_message(
                 "Interactive session for workflow {}"
-                " was successfully closed".format(workflow)
+                " was successfully closed".format(workflow),
+                msg_type="success",
             )
         except Exception as e:
             logging.debug(traceback.format_exc())
             logging.debug(str(e))
-            click.secho(
+            display_message(
                 "Interactive session could not be closed: \n{}".format(str(e)),
-                fg="red",
-                err=True,
+                msg_type="error",
             )
     else:
-        click.secho("Cannot find workflow {} ".format(workflow), fg="red", err=True)
+        display_message("Cannot find workflow {} ".format(workflow), msg_type="error")
