@@ -1,11 +1,12 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of REANA.
-# Copyright (C) 2017, 2018 CERN.
+# Copyright (C) 2017, 2018, 2019, 2020, 2021 CERN.
 #
 # REANA is free software; you can redistribute it and/or modify it
 # under the terms of the MIT License; see LICENSE file for more details.
 """REANA client utils."""
+
 import base64
 import datetime
 import json
@@ -14,20 +15,21 @@ import os
 import subprocess
 import sys
 import traceback
+from typing import NoReturn
 from uuid import UUID
 
 import click
-import yadageschemas
 import yaml
 from jsonschema import ValidationError, validate
 from reana_commons.errors import REANAValidationError
 from reana_commons.operational_options import validate_operational_options
 from reana_commons.serial import serial_load
+from reana_commons.snakemake import snakemake_load
+from reana_commons.yadage import yadage_load
 from reana_commons.utils import get_workflow_status_change_verb
+from reana_commons.workspaces import validate_workspace
 
 from reana_client.config import (
-    DOCKER_REGISTRY_INDEX_URL,
-    ENVIRONMENT_IMAGE_SUSPECTED_TAGS_VALIDATOR,
     reana_yaml_schema_file_path,
     reana_yaml_valid_file_names,
 )
@@ -39,54 +41,16 @@ from reana_client.validation.parameters import validate_parameters
 def workflow_uuid_or_name(ctx, param, value):
     """Get UUID of workflow from configuration / cache file based on name."""
     if not value:
-        click.echo(
-            click.style(
-                "Workflow name must be provided either with "
-                "`--workflow` option or with REANA_WORKON "
-                "environment variable",
-                fg="red",
-            ),
-            err=True,
+        display_message(
+            "Workflow name must be provided either with "
+            "`--workflow` option or with REANA_WORKON "
+            "environment variable",
+            msg_type="error",
         )
     else:
         return value
 
 
-def yadage_load(workflow_file, toplevel=".", **kwargs):
-    """Validate and return yadage workflow specification.
-
-    :param workflow_file: A specification file compliant with
-        `yadage` workflow specification.
-    :type workflow_file: string
-    :param toplevel: URL/path for the workflow file
-    :type toplevel: string
-
-    :returns: A dictionary which represents the valid `yadage` workflow.
-    """
-    schema_name = "yadage/workflow-schema"
-    schemadir = None
-
-    specopts = {
-        "toplevel": toplevel,
-        "schema_name": schema_name,
-        "schemadir": schemadir,
-        "load_as_ref": False,
-    }
-
-    validopts = {
-        "schema_name": schema_name,
-        "schemadir": schemadir,
-    }
-
-    try:
-        return yadageschemas.load(
-            spec=workflow_file, specopts=specopts, validopts=validopts, validate=True
-        )
-    except ValidationError as e:
-        e.message = str(e)
-        raise e
-
-
 def cwl_load(workflow_file, **kwargs):
     """Validate and return cwl workflow specification.
 
@@ -111,17 +75,23 @@ def load_workflow_spec(workflow_type, workflow_file, **kwargs):
         "yadage": yadage_load,
         "cwl": cwl_load,
         "serial": serial_load,
+        "snakemake": snakemake_load,
     }
-    """Dictionary to extend with new workflow specification loaders."""
 
-    return workflow_load[workflow_type](workflow_file, **kwargs)
+    """Dictionary to extend with new workflow specification loaders."""
+    load_function = workflow_load.get(workflow_type)
+    if load_function:
+        return load_function(workflow_file, **kwargs)
+    return {}
 
 
 def load_reana_spec(
     filepath,
+    access_token=None,
     skip_validation=False,
     skip_validate_environments=True,
     pull_environment_image=False,
+    server_capabilities=False,
 ):
     """Load and validate reana specification file.
 
@@ -137,29 +107,24 @@ def load_reana_spec(
             kwargs["specification"] = reana_yaml["workflow"].get("specification")
             kwargs["parameters"] = reana_yaml.get("inputs", {}).get("parameters", {})
             kwargs["original"] = True
-
         if "options" in reana_yaml.get("inputs", {}):
             try:
                 reana_yaml["inputs"]["options"] = validate_operational_options(
                     workflow_type, reana_yaml["inputs"]["options"]
                 )
             except REANAValidationError as e:
-                click.secho(e.message, err=True, fg="red")
+                display_message(e.message, msg_type="error")
                 sys.exit(1)
             kwargs.update(reana_yaml["inputs"]["options"])
+        if workflow_type == "snakemake":
+            kwargs["input"] = (
+                reana_yaml.get("inputs", {}).get("parameters", {}).get("input")
+            )
         return kwargs
 
     try:
         with open(filepath) as f:
             reana_yaml = yaml.load(f.read(), Loader=yaml.FullLoader)
-
-        workflow_type = reana_yaml["workflow"]["type"]
-        reana_yaml["workflow"]["specification"] = load_workflow_spec(
-            workflow_type,
-            reana_yaml["workflow"].get("file"),
-            **_prepare_kwargs(reana_yaml)
-        )
-
         if not skip_validation:
             display_message(
                 "Verifying REANA specification file... {filepath}".format(
@@ -168,6 +133,22 @@ def load_reana_spec(
                 msg_type="info",
             )
             _validate_reana_yaml(reana_yaml)
+        workflow_type = reana_yaml["workflow"]["type"]
+        reana_yaml["workflow"]["specification"] = load_workflow_spec(
+            workflow_type,
+            reana_yaml["workflow"].get("file"),
+            **_prepare_kwargs(reana_yaml)
+        )
+
+        if (
+            workflow_type == "cwl" or workflow_type == "snakemake"
+        ) and "inputs" in reana_yaml:
+            with open(reana_yaml["inputs"]["parameters"]["input"]) as f:
+                reana_yaml["inputs"]["parameters"] = yaml.load(
+                    f, Loader=yaml.FullLoader
+                )
+
+        if not skip_validation:
             validate_parameters(workflow_type, reana_yaml)
 
         if not skip_validate_environments:
@@ -177,11 +158,12 @@ def load_reana_spec(
             )
             validate_environment(reana_yaml, pull=pull_environment_image)
 
-        if workflow_type == "cwl" and "inputs" in reana_yaml:
-            with open(reana_yaml["inputs"]["parameters"]["input"]) as f:
-                reana_yaml["inputs"]["parameters"] = yaml.load(
-                    f, Loader=yaml.FullLoader
-                )
+        if server_capabilities:
+            root_path = reana_yaml.get("workspace", {}).get("root_path")
+            display_message(
+                "Verifying workspace in REANA specification file...", msg_type="info",
+            )
+            _validate_workspace(root_path, access_token)
 
         if workflow_type == "yadage":
             # We don't send the loaded Yadage workflow spec to the cluster as
@@ -212,12 +194,10 @@ def _validate_reana_yaml(reana_yaml):
     try:
         with open(reana_yaml_schema_file_path, "r") as f:
             reana_yaml_schema = json.loads(f.read())
-
             validate(reana_yaml, reana_yaml_schema)
         display_message(
             "Valid REANA specification file.", msg_type="success", indented=True,
         )
-
     except IOError as e:
         logging.info(
             "Something went wrong when reading REANA validation schema from "
@@ -285,14 +265,11 @@ def get_workflow_root():
             break
         else:
             if workflow_root == parent_dir:
-                click.echo(
-                    click.style(
-                        "Not a workflow directory (or any of the parent"
-                        " directories).\nPlease upload from inside"
-                        " the directory containing the reana.yaml "
-                        "file of your workflow.",
-                        fg="red",
-                    )
+                display_message(
+                    "Not a workflow directory (or any of the parent directories).\n"
+                    "Please upload from inside the directory containing "
+                    "the reana.yaml file of your workflow.",
+                    msg_type="error",
                 )
                 sys.exit(1)
             else:
@@ -306,12 +283,9 @@ def validate_input_parameters(live_parameters, original_parameters):
     parsed_input_parameters = dict(live_parameters)
     for parameter in parsed_input_parameters.keys():
         if parameter not in original_parameters:
-            click.echo(
-                click.style(
-                    "Given parameter - {0}, is not in reana.yaml".format(parameter),
-                    fg="red",
-                ),
-                err=True,
+            display_message(
+                "Given parameter - {0}, is not in reana.yaml".format(parameter),
+                msg_type="error",
             )
             del live_parameters[parameter]
     return live_parameters
@@ -348,13 +322,10 @@ def parse_secret_from_literal(literal):
     except ValueError as e:
         logging.debug(traceback.format_exc())
         logging.debug(str(e))
-        click.echo(
-            click.style(
-                'Option "{0}" is invalid: \n'
-                'For literal strings use "SECRET_NAME=VALUE" format'.format(literal),
-                fg="red",
-            ),
-            err=True,
+        display_message(
+            'Option "{0}" is invalid: \n'
+            'For literal strings use "SECRET_NAME=VALUE" format'.format(literal),
+            msg_type="error",
         )
 
 
@@ -378,18 +349,15 @@ def parse_secret_from_path(path):
     except FileNotFoundError as e:
         logging.debug(traceback.format_exc())
         logging.debug(str(e))
-        click.echo(
-            click.style(
-                "File {0} could not be uploaded: {0} does not exist.".format(path),
-                fg="red",
-            ),
-            err=True,
+        display_message(
+            "File {0} could not be uploaded: {0} does not exist.".format(path),
+            msg_type="error",
         )
 
 
 def get_api_url():
     """Obtain REANA server API URL."""
-    server_url = os.getenv("REANA_SERVER_URL", None)
+    server_url = os.getenv("REANA_SERVER_URL")
     return server_url.strip(" \t\n\r/") if server_url else None
 
 
@@ -397,23 +365,16 @@ def get_reana_yaml_file_path():
     """REANA specification file location."""
     matches = [path for path in reana_yaml_valid_file_names if os.path.exists(path)]
     if len(matches) == 0:
-        click.echo(
-            click.style(
-                "==> ERROR: No REANA specification file (reana.yaml) found. "
-                "Exiting.",
-                fg="red",
-            )
+        display_message(
+            "No REANA specification file (reana.yaml) found. Exiting.",
+            msg_type="error",
         )
         sys.exit(1)
     if len(matches) > 1:
-        click.echo(
-            click.style(
-                "==> ERROR: Found {0} REANA specification files ({1}). "
-                "Please use only one. Exiting.".format(
-                    len(matches), ", ".join(matches)
-                ),
-                fg="red",
-            )
+        display_message(
+            "Found {0} REANA specification files ({1}). "
+            "Please use only one. Exiting.".format(len(matches), ", ".join(matches)),
+            msg_type="error",
         )
         sys.exit(1)
     for path in reana_yaml_valid_file_names:
@@ -453,3 +414,34 @@ def run_command(cmd, display=True, return_output=False, stderr_output=False):
         if stderr_output:
             sys.exit(err.output.decode())
         sys.exit(err.returncode)
+
+
+def _validate_workspace(root_path: str, access_token: str) -> NoReturn:
+    """Validate workspace in REANA specification file.
+
+    :param root_path: workspace root path to be validated.
+    :param access_token: access token of the current user.
+
+    :raises ValidationError: Given workspace in REANA spec file does not validate against
+        allowed workspaces.
+    """
+    from reana_client.api.client import info
+
+    if not root_path:
+        display_message(
+            "Workspace not found in REANA specification file. Validation skipped.",
+            msg_type="warning",
+            indented=True,
+        )
+    else:
+        available_workspaces = (
+            info(access_token).get("workspaces_available", {}).get("value")
+        )
+        try:
+            validate_workspace(root_path, available_workspaces)
+            display_message(
+                "Workflow workspace appears valid.", msg_type="success", indented=True,
+            )
+        except REANAValidationError as e:
+            display_message(e.message, msg_type="error")
+            sys.exit(1)
