@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of REANA.
-# Copyright (C) 2017, 2018 CERN.
+# Copyright (C) 2018, 2019, 2020, 2021 CERN.
 #
 # REANA is free software; you can redistribute it and/or modify it
 # under the terms of the MIT License; see LICENSE file for more details.
@@ -52,6 +52,11 @@ def test_workflows_server_ok():
                 "user": "00000000-0000-0000-0000-000000000000",
                 "name": "mytest.1",
                 "id": "256b25f4-4cfb-4684-b7a8-73872ef455a1",
+                "size": {"raw": 0, "human_readable": "0 Bytes"},
+                "progress": {
+                    "run_started_at": "2018-06-13T09:47:40.28223",
+                    "run_finished_at": "2018-06-13T10:30:03.70303",
+                },
             }
         ]
     }
@@ -67,7 +72,9 @@ def test_workflows_server_ok():
             "reana_client.api.client.current_rs_api_client",
             make_mock_api_client("reana-server")(mock_response, mock_http_response),
         ):
-            result = runner.invoke(cli, ["list", "-t", reana_token])
+            result = runner.invoke(
+                cli, ["list", "-t", reana_token, "--include-progress"]
+            )
             message = "RUN_NUMBER"
             assert result.exit_code == 0
             assert message in result.output
@@ -87,6 +94,7 @@ def test_workflows_sorting():
                     "run_started_at": "2018-06-13T09:47:40.28223",
                     "run_finished_at": "2018-06-13T10:30:03.70303",
                 },
+                "size": {"raw": 0, "human_readable": "0 Bytes"},
             },
             {
                 "status": "running",
@@ -94,6 +102,7 @@ def test_workflows_sorting():
                 "user": "00000000-0000-0000-0000-000000000000",
                 "name": "mytest.2",
                 "id": "256b25f4-4cfb-4684-b7a8-73872ef455a2",
+                "size": {"raw": 0, "human_readable": "0 Bytes"},
             },
         ]
     }
@@ -134,7 +143,7 @@ def test_workflows_sessions():
                 "name": "workflow.1",
                 "session_type": "jupyter",
                 "session_uri": "/29136cd0-b259-4d48-8c1e-afe3572df408",
-                "size": "0",
+                "size": {"raw": 0, "human_readable": "0 Bytes"},
                 "status": "created",
                 "user": "00000000-0000-0000-0000-000000000000",
             }
@@ -168,7 +177,7 @@ def test_workflows_valid_json():
                 "user": "00000000-0000-0000-0000-000000000000",
                 "name": "mytest.1",
                 "id": "256b25f4-4cfb-4684-b7a8-73872ef455a1",
-                "size": "0K",
+                "size": {"raw": 0, "human_readable": "0 Bytes"},
             }
         ]
     }
@@ -185,16 +194,196 @@ def test_workflows_valid_json():
             make_mock_api_client("reana-server")(mock_response, mock_http_response),
         ):
             result = runner.invoke(cli, ["list", "-v", "-t", reana_token, "--json"])
+            assert result.exit_code == 0
+
+
+def test_workflows_include_progress():
+    """Test workflows command with --include-progress flag."""
+    response = {
+        "items": [
+            {
+                "status": "running",
+                "created": "2018-06-13T09:47:35.66097",
+                "user": "00000000-0000-0000-0000-000000000000",
+                "name": "mytest.1",
+                "id": "256b25f4-4cfb-4684-b7a8-73872ef455a1",
+                "size": {"raw": 0, "human_readable": "0 Bytes"},
+                "progress": {
+                    "total": {"job_ids": [], "total": 5},
+                    "running": {"job_ids": [], "total": 3},
+                    "finished": {"job_ids": [], "total": 2},
+                    "failed": {"job_ids": [], "total": 0},
+                },
+            }
+        ]
+    }
+    status_code = 200
+    mock_http_response, mock_response = Mock(), Mock()
+    mock_http_response.status_code = status_code
+    mock_response = response
+    env = {"REANA_SERVER_URL": "localhost"}
+    reana_token = "000000"
+    runner = CliRunner(env=env)
+    with runner.isolation():
+        with patch(
+            "reana_client.api.client.current_rs_api_client",
+            make_mock_api_client("reana-server")(mock_response, mock_http_response),
+        ):
+            result = runner.invoke(
+                cli, ["list", "--include-progress", "-t", reana_token]
+            )
+            assert result.exit_code == 0
+            assert "PROGRESS" in result.output
+            assert "2/5" in result.output
+
+
+def test_workflows_without_include_progress():
+    """Test workflows command without --include-progress flag."""
+    response = {
+        "items": [
+            {
+                "status": "running",
+                "created": "2018-06-13T09:47:35.66097",
+                "user": "00000000-0000-0000-0000-000000000000",
+                "name": "mytest.1",
+                "id": "256b25f4-4cfb-4684-b7a8-73872ef455a1",
+                "size": {"raw": 0, "human_readable": "0 Bytes"},
+                "progress": {
+                    "run_started_at": "2021-05-10T12:55:04",
+                    "run_finished_at": "2021-05-10T12:55:23",
+                },
+            }
+        ]
+    }
+    status_code = 200
+    mock_http_response, mock_response = Mock(), Mock()
+    mock_http_response.status_code = status_code
+    mock_response = response
+    env = {"REANA_SERVER_URL": "localhost"}
+    reana_token = "000000"
+    runner = CliRunner(env=env)
+    with runner.isolation():
+        with patch(
+            "reana_client.api.client.current_rs_api_client",
+            make_mock_api_client("reana-server")(mock_response, mock_http_response),
+        ):
+            result = runner.invoke(cli, ["list", "-t", reana_token])
+            assert result.exit_code == 0
+            assert "PROGRESS" not in result.output
+            assert "STARTED" in result.output
+            assert "2021-05-10T12:55:04" in result.output
+
+
+def test_workflows_include_workspace_size():
+    """Test workflows command with --include-workspace-size flag."""
+    response = {
+        "items": [
+            {
+                "status": "running",
+                "created": "2018-06-13T09:47:35.66097",
+                "user": "00000000-0000-0000-0000-000000000000",
+                "name": "mytest.1",
+                "id": "256b25f4-4cfb-4684-b7a8-73872ef455a1",
+                "size": {"human_readable": "15.97 MiB", "raw": 16741346},
+            }
+        ]
+    }
+    status_code = 200
+    mock_http_response, mock_response = Mock(), Mock()
+    mock_http_response.status_code = status_code
+    mock_response = response
+    env = {"REANA_SERVER_URL": "localhost"}
+    reana_token = "000000"
+    runner = CliRunner(env=env)
+    with runner.isolation():
+        with patch(
+            "reana_client.api.client.current_rs_api_client",
+            make_mock_api_client("reana-server")(mock_response, mock_http_response),
+        ):
+            result = runner.invoke(
+                cli, ["list", "--include-workspace-size", "-t", reana_token]
+            )
+            assert result.exit_code == 0
+            assert "SIZE" in result.output
+            assert "16741346" in result.output
+
+
+def test_workflows_without_include_workspace_size():
+    """Test workflows command without --include-workspace-size flag."""
+    response = {
+        "items": [
+            {
+                "status": "running",
+                "created": "2018-06-13T09:47:35.66097",
+                "user": "00000000-0000-0000-0000-000000000000",
+                "name": "mytest.1",
+                "id": "256b25f4-4cfb-4684-b7a8-73872ef455a1",
+                "size": {"human_readable": "", "raw": -1},
+            }
+        ]
+    }
+    status_code = 200
+    mock_http_response, mock_response = Mock(), Mock()
+    mock_http_response.status_code = status_code
+    mock_response = response
+    env = {"REANA_SERVER_URL": "localhost"}
+    reana_token = "000000"
+    runner = CliRunner(env=env)
+    with runner.isolation():
+        with patch(
+            "reana_client.api.client.current_rs_api_client",
+            make_mock_api_client("reana-server")(mock_response, mock_http_response),
+        ):
+            result = runner.invoke(cli, ["list", "-t", reana_token])
+            assert result.exit_code == 0
+            assert "SIZE" not in result.output
+
+
+def test_workflows_format():
+    """Test workflows command with --format."""
+    response = {
+        "items": [
+            {
+                "status": "running",
+                "created": "2018-06-13T09:47:35.66097",
+                "user": "00000000-0000-0000-0000-000000000000",
+                "name": "mytest.1",
+                "id": "256b25f4-4cfb-4684-b7a8-73872ef455a1",
+                "size": {"raw": 0, "human_readable": "0 Bytes"},
+            },
+            {
+                "status": "failed",
+                "created": "2018-06-14T09:47:35.66097",
+                "user": "00000000-0000-0000-0000-000000000000",
+                "name": "mytest.2",
+                "id": "256b25f4-4cfb-4684-b7a8-73872ef455a2",
+                "size": {"raw": 0, "human_readable": "0 Bytes"},
+            },
+        ]
+    }
+    status_code = 200
+    mock_http_response, mock_response = Mock(), Mock()
+    mock_http_response.status_code = status_code
+    mock_response = response
+    env = {"REANA_SERVER_URL": "localhost"}
+    reana_token = "000000"
+    runner = CliRunner(env=env)
+    _format = "status=failed"
+    with runner.isolation():
+        with patch(
+            "reana_client.api.client.current_rs_api_client",
+            make_mock_api_client("reana-server")(mock_response, mock_http_response),
+        ):
+            result = runner.invoke(
+                cli,
+                ["list", "-t", reana_token, "--json", '--format="{}"'.format(_format)],
+            )
             json_response = json.loads(result.output)
             assert result.exit_code == 0
             assert isinstance(json_response, list)
             assert len(json_response) == 1
-            assert "name" in json_response[0]
-            assert "run_number" in json_response[0]
-            assert "created" in json_response[0]
             assert "status" in json_response[0]
-            assert "id" in json_response[0]
-            assert "user" in json_response[0]
+            assert json_response[0]["status"] == "failed"
 
 
 def test_workflows_filter():
@@ -207,14 +396,8 @@ def test_workflows_filter():
                 "user": "00000000-0000-0000-0000-000000000000",
                 "name": "mytest.1",
                 "id": "256b25f4-4cfb-4684-b7a8-73872ef455a1",
-            },
-            {
-                "status": "failed",
-                "created": "2018-06-14T09:47:35.66097",
-                "user": "00000000-0000-0000-0000-000000000000",
-                "name": "mytest.2",
-                "id": "256b25f4-4cfb-4684-b7a8-73872ef455a2",
-            },
+                "size": {"raw": 340, "human_readable": "340 Bytes"},
+            }
         ]
     }
     status_code = 200
@@ -224,22 +407,21 @@ def test_workflows_filter():
     env = {"REANA_SERVER_URL": "localhost"}
     reana_token = "000000"
     runner = CliRunner(env=env)
-    filter_ = "status=failed"
+    filter = "status=running"
     with runner.isolation():
         with patch(
             "reana_client.api.client.current_rs_api_client",
             make_mock_api_client("reana-server")(mock_response, mock_http_response),
         ):
             result = runner.invoke(
-                cli,
-                ["list", "-t", reana_token, "--json", '--format="{}"'.format(filter_)],
+                cli, ["list", "-t", reana_token, "--json", "--filter", filter],
             )
             json_response = json.loads(result.output)
             assert result.exit_code == 0
             assert isinstance(json_response, list)
             assert len(json_response) == 1
             assert "status" in json_response[0]
-            assert json_response[0]["status"] == "failed"
+            assert "running" in json_response[0]["status"]
 
 
 def test_workflow_create_failed():
@@ -371,6 +553,7 @@ def test_workflow_start_follow(initial_status, final_status, exit_code):
         "id": "256b25f4-4cfb-4684-b7a8-73872ef455a1",
         "workflow_name": workflow_name,
         "user": "00000000-0000-0000-0000-000000000000",
+        "size": {"raw": 0, "human_readable": "0 Bytes"},
     }
     final_reponse = {
         "status": final_status,
@@ -378,6 +561,7 @@ def test_workflow_start_follow(initial_status, final_status, exit_code):
         "id": "256b25f4-4cfb-4684-b7a8-73872ef455a1",
         "workflow_name": workflow_name,
         "user": "00000000-0000-0000-0000-000000000000",
+        "size": {"raw": 340, "human_readable": "340 Bytes"},
     }
     initial_expected_message = get_workflow_status_change_msg(
         workflow_name, initial_status
@@ -419,11 +603,14 @@ def test_workflows_validate(create_yaml_workflow_schema):
     """Test validation of REANA specifications file."""
     message = "Valid REANA specification file"
     env = {"REANA_SERVER_URL": "localhost"}
+    reana_token = "000000"
     runner = CliRunner(env=env)
     with runner.isolated_filesystem():
         with open("reana.yaml", "w") as f:
             f.write(create_yaml_workflow_schema)
-        result = runner.invoke(cli, ["validate", "--file", "reana.yaml"],)
+        result = runner.invoke(
+            cli, ["validate", "-t", reana_token, "--file", "reana.yaml"],
+        )
         assert result.exit_code == 0
         assert message in result.output
 
@@ -595,7 +782,7 @@ def test_close_interactive_session():
             make_mock_api_client("reana-server")(mock_response, mock_http_response),
         ):
             result = runner.invoke(cli, ["close", "-t", reana_token, "-w", workflow])
-            assert expected_message == result.output
+            assert expected_message in result.output
 
 
 def test_multiple_specifications(create_yaml_workflow_schema):
@@ -616,10 +803,11 @@ def test_yml_ext_specification(create_yaml_workflow_schema):
     env = {"REANA_SERVER_URL": "localhost"}
     runner = CliRunner(env=env)
     message = "Valid REANA specification file"
+    reana_token = "000000"
     with runner.isolated_filesystem():
         with open("reana.yml", "w") as reana_schema:
             reana_schema.write(create_yaml_workflow_schema)
-        result = runner.invoke(cli, ["validate"])
+        result = runner.invoke(cli, ["validate", "-t", reana_token])
         assert result.exit_code == 0
         assert message in result.output
 
@@ -627,6 +815,6 @@ def test_yml_ext_specification(create_yaml_workflow_schema):
     with runner.isolated_filesystem():
         with open("reana.json", "w") as reana_schema:
             reana_schema.write(create_yaml_workflow_schema)
-        result = runner.invoke(cli, ["validate"])
+        result = runner.invoke(cli, ["validate", "-t", reana_token])
         assert result.exit_code != 0
         assert message in result.output
